"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseConf = void 0;
const assert_1 = require("assert");
const nginx_conf_1 = require("nginx-conf");
/**
 * Parses the given nginx config.
 */
function parseConf(source) {
    let result;
    // XXX: This function is actually synchronous, just silly API...
    nginx_conf_1.NginxConfFile.createFromSource(source, { tab: '  ' }, (err, conf) => {
        if (err)
            throw err;
        result = conf;
    });
    assert_1.strict(result, 'This should not have happened');
    return nginxConfEditor(result);
}
exports.parseConf = parseConf;
const nginxConfEditor = (conf) => ({
    get(path) {
        const item = get(conf.nginx, path);
        return Array.isArray(item) ? item.map(x => x._value) : item === null || item === void 0 ? void 0 : item._value;
    },
    applyPatch(patch) {
        for (const op of patch) {
            applyOperation(conf.nginx, op);
        }
        return this;
    },
    toString() {
        return conf.toString();
    },
});
function applyOperation(confRoot, operation) {
    const splitPath = operation.path.split('/');
    const itemName = splitPath.pop();
    const parentPath = splitPath.join('/');
    const parent = get(confRoot, parentPath);
    if (!parent) {
        if (operation.op !== 'remove') {
            throw RangeError(`Directive at ${parentPath} does not exist`);
        }
        return;
    }
    switch (operation.op) {
        case 'add':
            parent._add(itemName, operation.value);
            break;
        case 'default':
            if (get(confRoot, operation.path) == null) {
                parent._add(itemName, operation.value);
            }
            break;
        case 'remove':
            // NOTE: ConfItem._remove() does not remove multi-values.
            delete parent[itemName];
            break;
        case 'set':
            if (parent[itemName]) {
                delete parent[itemName];
            }
            parent._add(itemName, operation.value);
    }
}
function get(confRoot, path) {
    const pointer = path === null || path === void 0 ? void 0 : path.split('/');
    if (!pointer || pointer[0] !== '') {
        throw Error(`Invalid JSON pointer: ${path}`);
    }
    const len = pointer.length;
    if (len === 1) {
        return confRoot;
    }
    for (let i = 1, item = confRoot; i < len; i++) {
        const p = pointer[i];
        // This is a hacky workaround.
        // TODO: Remove after https://github.com/tmont/nginx-conf/pull/27 is merged.
        if (!(p in item)) {
            if (p === '0') {
                continue;
            }
            if (!/d+/.test(p) && typeof item[0] === 'object') {
                item = item[0];
            }
        }
        item = item[p];
        if (i === len - 1) {
            return item;
        }
        if (typeof item !== 'object') {
            return undefined;
        }
    }
    return undefined;
}
//# sourceMappingURL=nginxConf.js.map