"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nginxVersionInfo = void 0;
const anylogger_1 = __importDefault(require("anylogger"));
const execa_1 = __importDefault(require("execa"));
const log = anylogger_1.default('nginx-testing');
/**
 * Executes the nginx binary `nginxBinPath` with option `-V` and returns parsed
 * version and info about the modules it was compiled with(out).
 */
async function nginxVersionInfo(nginxBinPath) {
    log.debug(`Executing '${nginxBinPath} -V'`);
    const { stderr, exitCode } = await execa_1.default(nginxBinPath, ['-V'], { timeout: 500 });
    if (exitCode > 0) {
        throw Error(`'${nginxBinPath} -V' failed with code ${exitCode}: ${stderr}`);
    }
    log.debug(`nginx -V: ${stderr}`);
    return parseOutput(stderr);
}
exports.nginxVersionInfo = nginxVersionInfo;
function parseOutput(output) {
    var _a, _b;
    const [, version = ''] = (_a = /^nginx version: nginx\/(\S+)/m.exec(output)) !== null && _a !== void 0 ? _a : [];
    const [, config = ''] = (_b = /^configure arguments: (.*)/m.exec(output)) !== null && _b !== void 0 ? _b : [];
    const modules = [...config.matchAll(/--(with(?:out)?)-(\w+)_module(?:=(\w+))?\b/g)]
        .reduce((acc, m) => {
        acc[m[2]] = m[1] + (m[3] === 'dynamic' ? `-${m[3]}` : '');
        return acc;
    }, {});
    return {
        version,
        modules,
    };
}
//# sourceMappingURL=nginxVersionInfo.js.map